#include "stdafx.h"
#include "convert.h"

IplImage* convertToColor(IplImage* src) {

  IplImage *c= cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,3);
  
  iplGrayToColor(src,c,1,1,1);

  return c;
}

IplImage* convertToImage(IplImage* src) {

   IplImage *res= cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,1);
   unsigned char* imu= reinterpret_cast<unsigned char *>(res->imageData);
   float* imf= reinterpret_cast<float *>(src->imageData);

   int nl= src->height;
   int nc= src->width;
   int step= res->widthStep;
   int stepf= src->widthStep/sizeof(float);

   if (src->depth == IPL_DEPTH_32F) {

     for (int i=0; i<nl; i++) {

       for (int j=0; j<nc; j++) {

		    imu[j]= imf[j]> 255.0 ? 255 : static_cast<int>(imf[j]+0.5);
       }

       imf+= stepf;
       imu+= step;
     }

	 return res;

   } else {

       cvReleaseImage(&res);
	   return 0;
   }
}

IplImage* convertToFloat(IplImage* src) {

   IplImage *res= cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_32F,1);
   float* imf= reinterpret_cast<float *>(res->imageData);
   unsigned char* imu= reinterpret_cast<unsigned char *>(src->imageData);

   int nl= src->height;
   int nc= src->width;
   int step= src->widthStep;
   int stepf= res->widthStep/sizeof(float);

   if (src->depth == IPL_DEPTH_8U) {

     for (int i=0; i<nl; i++) {

       for (int j=0; j<nc; j++) {

    		 imf[j]= static_cast<float>(imu[j]);
       }

       imf+= stepf;
       imu+= step;
     }

	 return res;

   } else {

        cvReleaseImage(&res);
	    return 0;
   }
}

IplImage* threshold(IplImage* src, unsigned int th, unsigned char val) {

   IplImage *res= cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,1);
   unsigned char* imu= reinterpret_cast<unsigned char *>(res->imageData);

   int nl= res->height;
   int nc= res->width;
   int step= res->widthStep;

   if (src->depth == IPL_DEPTH_8U) {

     unsigned char* im= reinterpret_cast<unsigned char *>(src->imageData);
     int stepi= src->widthStep;

     for (int i=0; i<nl; i++) {

       for (int j=0; j<nc; j++) {

  	    if (static_cast<unsigned int>(im[j]) > th) {

	        imu[j]= val;

		} else {

	        imu[j]= 0;
		}
       }

       imu+= step;
       im+= stepi;
     }
     
	   return res;

   } else if (src->depth == IPL_DEPTH_32F) {

     float* imf= reinterpret_cast<float *>(src->imageData);
     int stepf= src->widthStep/sizeof(float);

     for (int i=0; i<nl; i++) {

       for (int j=0; j<nc; j++) {

  	    if (imf[j] > th) {

			imu[j]= val;

		} else {

	        imu[j]= 0;
		}
       }

       imu+= step;
       imf+= stepf;
     }

	   return res;

   } else {

       cvReleaseImage(&res);
	   return 0;
   }
}

IplImage* combine(IplImage* src, IplImage* add) {

   IplImage *res= cvCreateImage(cvSize(src->width,src->height),IPL_DEPTH_8U,1);
   unsigned char* imu= reinterpret_cast<unsigned char *>(res->imageData);
   unsigned char* ima= reinterpret_cast<unsigned char *>(add->imageData);
   unsigned char* im= reinterpret_cast<unsigned char *>(src->imageData);

   int nl= res->height;
   int nc= res->width;
   int step= res->widthStep;
   int stepa= add->widthStep;
   int steps= src->widthStep;

   if (src->depth == IPL_DEPTH_8U) {


     for (int i=0; i<nl; i++) {

       for (int j=0; j<nc; j++) {

  	    if (ima[j] != 0) {

            imu[j]= ima[j];

        } else {

	        imu[j]= im[j];
        }
       }

       ima+= stepa;
       imu+= step;
       im+= steps;
     }
     
  	return res;

   } else {

       cvReleaseImage(&res);
	   return 0;
   }
}
