
/*
//
//               INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//        Copyright (c) 1998-2000 Intel Corporation. All Rights Reserved.
//
//  VSS
//    $Workfile:: create.cpp            $
//    $Revision:: 4                     $
//    $Modtime:: 3.05.98 16:59          $
//    $Nokeywords:                      $
//
//  Created: 20.04.98 7:43:13 by chan
//
*/

#include <stdio.h>
#include <math.h>
#include <conio.h>
#include <ipl.h>
#include <vector>

#include "macros.inc"
#include "cookroom.h"
#include "cookdraw.h"
#include "convert.h"
#include "draw.h"
#include "judoca.h"
#include "Derivatives.h"

static const bool is_modal = true;


/// 

/* //////////////////////////////////////////////////////////////////////
// Name:       create_manual_gray
// Purpose:    to show how to gain access to image data
// Context:    IPLib tutorial
// Returns:    void
// Parameters: void
//
//  Notes:     An example how to create a gray image manually
//             and how to gain access to image data
//          
//             Useless for derivative program, but this is a nice example.
*/

void create_manual_gray( int width = 128, int height = 100 ) {

   IplImage* img = iplCreateImageHeader( 
      1,                            // 1 channel
      0,                            // no alpha channel
      IPL_DEPTH_16U,                // data type is ushort
      "GRAY",                       // color model
      "GRAY",                       // channel sequence
      IPL_DATA_ORDER_PIXEL,         // channel arrangement
      IPL_ORIGIN_TL,                // top left orientation
      IPL_ALIGN_DWORD,              // 4 bytes align
      width, height,                // image width and height
      NULL, NULL,                   // no ROI, no mask ROI
      NULL, NULL);                  // image ID, not tiled

   /// allocate image data without filling
   iplAllocateImage( img, 0, 0 );   

   /// draw wedge if image was created
   if( img->imageData ) {
      /// factor to get full range of pixel values
      int const muler = 65600 / (width + height);
      char* p = img->imageData;
      /// set every pixel value
      for( int y=0; y<height; ++y ) {
         for( int x=0; x<width; ++x )
            /// pixel is of 16u depth
            ((unsigned short*)p)[x] = muler * (x + y);
         /// go to next line of the image
         p = p + img->widthStep;
      }
      /// disply the image
      ipView( img, " Manual Gray", is_modal );
   }
   /// free memory
   iplDeallocate( img, IPL_IMAGE_ALL );
}

/// How to use? What parameters can be specified

char* getUsageStr( char* msg ) {
   sprintf( msg, "%s", "derive [-tn] [-wn] image.bmp\n"
      "   -tn n (integer) is the threshold applied to the edge strenght (default 10)\n"
      "   -sf f (float) is the variance of the gaussian derivative filter (default 1.0)\n"
      "   image.bmp is the name of a BMP gray level image (run-length encoded)\n");
   return msg;
}

int usage() {
   char msg[1024];
   ::MessageBox( NULL, getUsageStr( msg ), "Help", MB_OK | MB_ICONINFORMATION );
   return 1;
}

/* //////////////////////////////////////////////////////////////////////
// Name:       main
// Purpose:    to run IPLib lesson one examples
// Context:    IPLib tutorial
// Returns:    0 if has been executed, 
               2 if cosole message about usage was printed
// Parameters: 
      int argc - num of parameters > 1
      char** argv - arguments strings
//
//  Notes:     it is actual for console application only
*/

int WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR lpCmdLine, int ) {

   char line[256];                     /// to copy command line
   char* argv[10];
   
   /// simulate argc and argv parameters
   int argc = parseCmndLine( "create.exe", lpCmdLine, line, 256, argv, 10 );

   if (argc <= 1) {
     return usage();
   }

   int thresh= 10;
   float variance= 1.0;
   char imagename[120];

   // get options
   for (int a= 1; a<argc; a++) {  

     if (sscanf(argv[a], "-t%d", &thresh) == 1)
       ;
     else if (sscanf(argv[a], "-s%f", &variance) == 1)
       ;
     else if (sscanf(argv[a], "-%s", &imagename) == 1) {

       return usage();

     } else {
       sscanf(argv[a], "%s", imagename);
     }
   }

   BITMAPINFOHEADER* bmphdr = ipLoad(imagename); // Cookroom.lib function
   if( !bmphdr ) {
     ::MessageBox( NULL, "Problem to read bmp-file", "Error", MB_OK | MB_ICONINFORMATION );
	 return 1;
   }

   int w= bmphdr->biWidth;
   int h= bmphdr->biHeight;
   int np= w*h;

   // read input image
   IplImage* img = iplCreateImageHeader( 
         1,                            // number of channels
         0,                            // alpha channel
         IPL_DEPTH_8U,                // data of byte type 
         "GRAY", "GRAY",               // color model and channel seq (in color: "RGB", "BGR")
         IPL_DATA_ORDER_PIXEL,         // channel arrangement
         IPL_ORIGIN_BL,                // top left orientation
         IPL_ALIGN_QWORD,              // 8 bytes align for optimal MMX performances
         bmphdr->biWidth, bmphdr->biHeight,
         NULL,                         // ROI
         NULL, NULL, NULL );           // no mask ROI, image ID, not tiled

   if( !img ) {

     ::MessageBox( NULL, "Problem to create image", "Error", MB_OK | MB_ICONINFORMATION );
     return 1;
   }

   iplConvertFromDIB( bmphdr, img );
   free( bmphdr );

   IplImage* imgf= convertToFloat(img); //  iplConvert(img, imgf); ?? does not work ??

   /* to display the circle and the junction edges
   int npt;
   int**c= getCirclePoints(11., &npt);
   int*** pt= getLinePoints(c, npt, 11);
   char ttt[100];
   sprintf(ttt,"npt=%d",npt);
   ::MessageBox( NULL, ttt, "Error", MB_OK | MB_ICONINFORMATION );
   char pix[3]={255,255,255};

   int g=0;
   for (int bx=0; bx<8; bx++)
    for (int by=0; by<8; by++){
      for (int x=0; x<npt; x++) {
        iplPutPixel(img, 100+bx*50+c[0][x] , 100+by*50+c[1][x], &pix);
      }
      for (int xx=0; xx< 11 && (pt[0][g][xx]!=0 || pt[1][g][xx]!=0); xx++) {
        iplPutPixel(img, 100+bx*50+pt[0][g][xx] , 100+by*50+pt[1][g][xx], &pix);
      }
      g++;
    }
   */

   // display original image
   ipView( img, imagename, false);
		 
   // compute gaussuian derivatives
   IplImage* dx = derivex(imgf,variance);	
   IplImage* dy = derivey(imgf,variance);	
   IplImage* edge= norm(dx,dy);

   // display edge image
   IplImage* edgeImage= convertToImage(edge);
   ipView( edgeImage, "Edges", false);
   iplDeallocate( edgeImage, IPL_IMAGE_ALL );

   // apply threshold
   IplImage* edgemap= threshold(edge, thresh);

   // display binary edge map
   ipView( edgemap, "Binary Edge Map", false);

   // non max suppression
   IplImage* edgeMax= nonMax(edge, dx, dy);
   IplImage* edgeMaxBin= threshold(edgeMax, thresh);

   // display binary max edge map
   ipView( edgeMaxBin, "Max Binary Edge Map", false);

   // get junctions
   std::vector<Junction> junctions;
   IplImage* junction= judoca(edgemap, edgeMaxBin, 11, 8, junctions);
   ipView( junction, "JUDOCA junctions", false);

   // save junctions
   saveJunctions("test.junctions", junctions);

   // display junctions
//   IplImage* detected= combine(img, junction);
//   ipView( detected, "Junctions on image", true);
   drawJunctions(img,junctions,15);
   ipView( img, "Junctions on image", true);

   // deallocations
   iplDeallocate( img, IPL_IMAGE_ALL );
   iplDeallocate( junction, IPL_IMAGE_ALL );
   iplDeallocate( edgeMaxBin, IPL_IMAGE_ALL );
   iplDeallocate( edgemap,    IPL_IMAGE_ALL );
   iplDeallocate( imgf,       IPL_IMAGE_ALL );
   iplDeallocate( dx,         IPL_IMAGE_ALL );
   iplDeallocate( dy,         IPL_IMAGE_ALL );
   iplDeallocate( edge,       IPL_IMAGE_ALL );
   iplDeallocate( edgeMax,    IPL_IMAGE_ALL );

   return 0;
}
