#include "stdafx.h"
#include "Derivatives.h"
#include <math.h>

IplImage* derivex(IplImage* src, float sigma){

    IplImage* dst = iplCreateImageHeader(1,0,IPL_DEPTH_32F,"GRAY","G",IPL_DATA_ORDER_PIXEL,IPL_ORIGIN_TL,IPL_ALIGN_QWORD,src->width,src->height,NULL,NULL,NULL,NULL);
    iplAllocateImageFP(dst, 0, 0);

    int dim= 1+2*static_cast<int>(sigma*3.0f+0.5f);

    float* g = new float[dim*dim];

	  sigma *= sigma;

    float e = -1.0f/(2.0f*sigma);
    float d = -1.0f/(6.2831853f*sigma*sigma);

    for (int i=0, x= -dim/2; i<dim; i++, x++)
    	for (int j=0, y= -dim/2; j<dim; j++, y++) {
        	g[i+j*dim]= d*x*static_cast<float>(exp(e*(x*x+y*y)));
        }

    IplConvKernelFP* ker = iplCreateConvKernelFP(dim,dim,dim/2,dim/2,g);
	iplConvolve2DFP(src,dst,&ker,1,0);

    iplDeleteConvKernelFP(ker);

    return dst;
}

IplImage* derivey(IplImage* src, float sigma){

    IplImage* dst = iplCreateImageHeader(1,0,IPL_DEPTH_32F,"GRAY","G",IPL_DATA_ORDER_PIXEL,IPL_ORIGIN_TL,IPL_ALIGN_QWORD,src->width,src->height,NULL,NULL,NULL,NULL);
    iplAllocateImageFP(dst, 0, 0);

    int dim= 1+2*static_cast<int>(sigma*3.0f+0.5f);

    float* g = new float[dim*dim];

	  sigma *= sigma;

    float e = -1.0f/(2.0f*sigma);
    float d = -1.0f/(6.2831853f*sigma*sigma);

    for (int i=0, x= -dim/2; i<dim; i++, x++)
    	for (int j=0, y= -dim/2; j<dim; j++, y++) {
        	g[i+dim*j]= d*y*static_cast<float>(exp(e*(x*x+y*y)));
        }

    IplConvKernelFP* ker = iplCreateConvKernelFP(dim,dim,dim/2,dim/2,g);
	iplConvolve2DFP(src,dst,&ker,1,0);

    iplDeleteConvKernelFP(ker);

    return dst;
}

IplImage* norm(IplImage* dx, IplImage* dy) {

   IplImage* res = iplCreateImageHeader(1,0,IPL_DEPTH_32F,"GRAY","G",IPL_DATA_ORDER_PIXEL,IPL_ORIGIN_TL,IPL_ALIGN_QWORD,dx->width,dy->height,NULL,NULL,NULL,NULL);
   iplAllocateImageFP(res, 0, 0);

   float* imr= reinterpret_cast<float *>(res->imageData);
   float* imx= reinterpret_cast<float *>(dx->imageData);
   float* imy= reinterpret_cast<float *>(dy->imageData);

   int stepr= res->widthStep/sizeof(float);
   int stepx= dx->widthStep/sizeof(float);
   int stepy= dy->widthStep/sizeof(float);

   int height= dx->height;
   int width= dx->width; 

   for (int i=0; i<height; i++) {
     for (int j=0; j<width; j++) {

      imr[j]= static_cast<float>(fabs(imx[j]) + fabs(imy[j]));
     }

     imr+= stepr;
     imx+= stepx;
     imy+= stepy;
   }

   return res;
}

IplImage* nonMax(IplImage* src, IplImage* dx, IplImage* dy) {

   IplImage *res = iplCloneImage( src );

   int nbl= src->imageSize / src->height;     // bytes per line
   int nfl= src->imageSize / src->height / 4; // float per line
   int np=  src->width * src->height;
   int nl=  src->height;

   float* datasrc= reinterpret_cast<float *>(src->imageData);
   float* datax=   reinterpret_cast<float *>(dx->imageData);
   float* datay=   reinterpret_cast<float *>(dy->imageData);
   float* datares= reinterpret_cast<float *>(res->imageData);

   float* up;
   float* cur;
   float* down;

   float* curx;
   float* cury;

   float* out;

   int nc= src->width - 2;

   for (int j=0; j<(nl-2); j++) {

     up=      datasrc + j*nfl + 1;
     cur=     datasrc + (j+1)*nfl + 1;
     down=    datasrc + (j+2)*nfl + 1;
     curx=    datax + (j+1)*nfl + 1;
     cury=    datay + (j+1)*nfl + 1;
     out=     datares + (j+1)*nfl + 1;
     
     for (int i=0; i<nc-1; i++) {

   	   if (fabs(*cury) < 0.4142*fabs(*curx)) {

         if (*cur <= *(cur-1) || *cur < *(cur+1))
           *out = 0;

       } else if (fabs(*curx) < 0.4142*fabs(*cury)) {

         if (*cur <= *down || *cur < *up)
           *out = 0;

       } else if (*curx * *cury > 0) {
            	
	     if (*cur <= *(up-1) || *cur < *(down+1))
           *out = 0;

       } else {
       
	     if (*cur <= *(up+1) || *cur < *(down-1))
           *out = 0;
       }

       up++, down++, cur++;
       curx++, cury++;
       out++;
     }
   }

   return res;
}