
#include "stdafx.h"
#include "judoca.h"

int** getCirclePoints(double r, int* np) {

  int x,y;
  int ri= static_cast<int>(r+0.5);
  int p= static_cast<int>(r/sqrt(2.) + 1.);

  int **c= new int*[2];
  c[0]= new int[8*p];
  c[1]= new int[8*p];

  c[0][0]= ri;
  c[1][0]= 0;
  c[0][1]= -ri;
  c[1][1]= 0;
  c[1][2]= ri;
  c[0][2]= 0;
  c[1][3]= -ri;
  c[0][3]= 0;

  int k= 4;

  for (x=1; x<=p; x++) {

    c[0][k]= x;
    c[1][k++]= y= static_cast<int>(sqrt(r*r - x*x) + 0.5);

    c[0][k]= -x;
    c[1][k++]= y;

    c[0][k]= x;
    c[1][k++]= -y;

    c[0][k]= -x;
    c[1][k++]= -y;
  }

  for (y--; y>0; y--) {

    c[1][k]= y;
    c[0][k++]= x= static_cast<int>(sqrt(r*r - y*y) + 0.5);

    c[1][k]= -y;
    c[0][k++]= x;

    c[1][k]= y;
    c[0][k++]= -x;

    c[1][k]= -y;
    c[0][k++]= -x;
  }

  *np= k;

  return c;
}

void deleteCirclePoints(int **c) {

	delete[] c[0];
	delete[] c[1];
	delete[] c;
}

int**** getLinePoints(int **c, int np, int r) {

  double slope;
 
  // c[x or y][pt on circle][pt on line]
  int ****pt= new int***[2];
  pt[0]= new int**[np];
  pt[1]= new int**[np];

  for (int i=0; i<np; i++) {

    // find points between (c[0][i],c[1][i]) and (0,0)
    pt[0][i]= new int*[r];
    pt[1][i]= new int*[r];
    for (int w=0; w<r; w++) {
      pt[0][i][w]= new int[4];
	  pt[0][i][w][0]= pt[0][i][w][1]= 0;
      pt[1][i][w]= new int[4];
	  pt[1][i][w][0]= pt[1][i][w][1]= 0;
    }

	if (c[0][i] == 0) 
		slope = 1000;
	else
        slope= ((double)(c[1][i]))/((double)(c[0][i]));

    int xxx,yyy;
    xxx=c[0][i];
    yyy=c[1][i];

    int dx, dy;
    double dxf, dyf;
    int x,y;
    double xf,yf;
    double arr;
    int t;
    t=0;

    if (fabs(slope) < 1.0) {

      dx= (c[0][i] < 0) ? -1 : 1;
      dyf= (c[1][i] < 0) ? -fabs(slope) : fabs(slope);
      arr= (c[1][i] < 0) ? -1. : 1.;

      for (x=dx, yf=dyf; x!=c[0][i]; x+= dx, yf+= dyf) {
       
        pt[0][i][t][0]= x;
        pt[0][i][t][1]= x;
        pt[1][i][t][0]= static_cast<int>(yf);
        pt[1][i][t][1]= static_cast<int>(yf+arr);

		// Distance from point to line
        pt[0][i][t][2]= static_cast<int>(100-100*(slope*pt[0][i][t][0]-pt[1][i][t][0])*(slope*pt[0][i][t][0]-pt[1][i][t][0])/(1+slope*slope));
        pt[0][i][t][3]= static_cast<int>(100-100*(slope*pt[0][i][t][1]-pt[1][i][t][1])*(slope*pt[0][i][t][1]-pt[1][i][t][1])/(1+slope*slope));
		t++;
      }

    } else {

      dy= (c[1][i] < 0) ? -1 : 1;
      dxf= (c[0][i] < 0) ? -fabs(1./slope) : fabs(1./slope);
      arr= (c[0][i] < 0) ? -1. : 1.;

      for (xf=dxf, y=dy; y!=c[1][i]; y+= dy, xf+= dxf) { 

        pt[1][i][t][0]= y;
        pt[1][i][t][1]= y;
        pt[0][i][t][0]= static_cast<int>(xf);
        pt[0][i][t][1]= static_cast<int>(xf+arr);

        pt[0][i][t][2]= static_cast<int>(100-100*(slope*pt[0][i][t][0]-pt[1][i][t][0])*(slope*pt[0][i][t][0]-pt[1][i][t][0])/(1+slope*slope));
        pt[0][i][t][3]= static_cast<int>(100-100*(slope*pt[0][i][t][1]-pt[1][i][t][1])*(slope*pt[0][i][t][1]-pt[1][i][t][1])/(1+slope*slope));
		t++;
      }
    }
  }

  return pt;
}

void deleteLinePoints(int**** pt, int np, int r) {

	for (int i=0; i<np; i++) {
     for (int w=0; w<r; w++) {

      delete[] pt[0][i][w];
      delete[] pt[1][i][w];
	 }
    }

	for (i=0; i<np; i++) {

	  delete[] pt[0][i];
      delete[] pt[1][i];
	}
	
	delete[] pt;
}

/*
 * This is the function that accomplishes the junction detection.
 * IplImage* B     : the binary image edge image
 * IplImage* Bp    : the binary image edge image after nonmaxima suppression
 * int radius      : the length of the junction edges
 * float sj        : the minimal acceptable strength for a junction edge
 * float maxangle2 : the maximal allowed angle between the edges of a 2-junction
 * std::vector<Junction> &junctions : the vector containing the junctions (output)
 */
IplImage* judoca(IplImage* B, IplImage* Bp, int radius, float sj, float minang, float maxangle2, std::vector<Junction> &junctions) {

   IplImage* res = iplCreateImageHeader(1,0,IPL_DEPTH_8U,"GRAY","G",IPL_DATA_ORDER_PIXEL,IPL_ORIGIN_TL,IPL_ALIGN_QWORD,B->width,B->height,NULL,NULL,NULL,NULL);
   iplAllocateImage(res, 1, 0);

   int np= B->width * B->height;
   int nl= B->height;
   int nc= B->width;
   int step= B->widthStep;

   char* b=   B->imageData;
   char* bp=  Bp->imageData;
   char* result= res->imageData;

   // get circle points and edge points
   int npts;
   int** c= getCirclePoints((double)radius, &npts);
   int**** pt= getLinePoints(c, npts, radius);

   double* angles= new double[npts];  // compute corresponding angles
   for (int r=0; r<npts; r++) {

     angles[r]= atan2(c[1][r], c[0][r]);
   }

   Junction junction(npts);  // will contain each junction found
   int* anchors= new int[npts];
   double* astr= new double[npts];
   double* ajunction= new double[npts];
   int na;
   double angmax1= 3.1415926*(maxangle2)/180; // 2-juntion angle invalid interval
   double angmax2= 3.1415926*(360-maxangle2)/180; 
   double angmin1= 3.1415926*(minang)/180; // min angle invalid interval
   double angmin2= 3.1415926*(360-minang)/180; 
   
   for (int j=radius+1; j<nl-radius-1; j++) {
     for (int i=radius+1; i<nc-radius-1; i++) {

       // if in B
       if (*(b+j*step+i) != 0) {

         // scan circle
         na= 0;
         for (int k=0; k<npts; k++) {

           // if in B+
           if (*(bp+(j+c[1][k])*step+i+c[0][k]) != 0) {

             anchors[na++]= k;
           }
         }

         if (na < 2) 
           continue;

         if (na == 2 && fabs(angles[anchors[0]]-angles[anchors[1]]) > angmax1
                     && fabs(angles[anchors[0]]-angles[anchors[1]]) < angmax2)
           continue;

         // scan the junction edges
         for (int je=0; je<na; je++) {

           astr[je]= 0;
           for (int xx=0; xx< radius && 
                           (pt[0][anchors[je]][xx][0]!=0 || pt[1][anchors[je]][xx][0]!=0); xx++) {

             // the edge point is not on B
             if (*(b+(j+pt[1][anchors[je]][xx][0])*step+i+pt[0][anchors[je]][xx][0]) == 0 &&
                 *(b+(j+pt[1][anchors[je]][xx][1])*step+i+pt[0][anchors[je]][xx][1]) == 0) {

               astr[je]= -1;
               break;
             }

             // if the edge point is on B+
			 int dstr;
			 dstr= 0;

             if (*(bp+(j+pt[1][anchors[je]][xx][0])*step+i+pt[0][anchors[je]][xx][0]) != 0) {

			     dstr= pt[0][anchors[je]][xx][2];
             }

             if (*(bp+(j+pt[1][anchors[je]][xx][1])*step+i+pt[0][anchors[je]][xx][1]) != 0) {

				 dstr= (pt[0][anchors[je]][xx][3] > dstr) ? pt[0][anchors[je]][xx][3] : dstr;
             }

             astr[je]+= dstr;

           } // for (int xx=0; xx< radius && ...

           astr[je]/= 100*xx; // convert strength in percentage

         } // for (int je=0; je<na; je++)

         // check junction edges found
         int njunction;
         double sjmin;

         njunction= 0;
         sjmin= 1.0;

         for (je=0; je<na; je++) {

           // if the strenght is greater than threshold sj
           if (astr[je] > sj) {

             astr[njunction]= astr[je];
             ajunction[njunction++]= angles[anchors[je]];

             // get the junction strength
             if (astr[je] < sjmin)
               sjmin= astr[je];
           }
         }


         for (je=0; je < (njunction-1); je++){ // to check if angle between edges is too small

           if (ajunction[je] > 9999)
             continue;

           for (int j=je+1; j<njunction; j++) {

             if (fabs(ajunction[je]-ajunction[j]) < angmin1 ||
                 fabs(ajunction[je]-ajunction[j]) > angmin2) { 

               if (astr[j] < astr[je]) {

                 ajunction[j]= 10000;

               } else {

                 ajunction[je]= 10000;
                 break;

               }
             }  
           }
         }

         na= njunction;
         njunction= 0;

         for (je=0; je<na; je++) {

           if (ajunction[je] < 9999) {

             astr[njunction]= astr[je];
             ajunction[njunction++]= ajunction[je];
           }
         }

         if (njunction == 2) { // check if it is a valid 2-junction

           if (fabs(ajunction[0]-ajunction[1]) > angmax1 && 
               fabs(ajunction[0]-ajunction[1]) < angmax2 ) { // a line not a 2-junction

             njunction= 0;
           }
         }

         if (njunction > 1) { // we have a junction
			 *(result+j*step+i)= 100+static_cast<int>(0.5+sjmin*100) + (njunction>2?50:0);
            junction.n= njunction;
            junction.x= i;
            junction.y= j;

            for (int q=0; q<njunction; q++) {

              junction.orientation[q]= ajunction[q];
              junction.strenght[q]= astr[q];
            }

            junctions.push_back(junction);
         }

       } // if (*(b+j*step+i) != 0)
     }  // for (int i=0; i<nc; i++)
   }   // for (int j=0; j<nl; j++)

   deleteCirclePoints(c);
   deleteLinePoints(pt, npts, radius);
   delete[] angles;
   delete[] anchors;
   delete[] astr;
   delete[] ajunction;

   return res;
}


void nonMaxSupp(IplImage* junction_image, std::vector<Junction> &junctions, int radius) {

	   unsigned char* im= reinterpret_cast<unsigned char *>(junction_image->imageData);
       int step= junction_image->widthStep;
	   int x,y;
	   bool tested;

	   for (int j=0; j<junctions.size(); j++) {

		   x= junctions[j].x;
		   y= junctions[j].y;
		   tested= false;

           for (int dy= -radius; !tested && dy <= radius; dy++) {

			   if (y+dy < 0 || y+dy >= junction_image->height)
				   continue;

			   for (int dx= -radius; !tested && dx <= radius; dx++) {

				   if (x+dx < 0 || x+dx >= junction_image->width)
						continue;

				   if (dx==0 && dy==0)
						continue;

			       int s,ss;
				   s= *(im+y*step+x);
				   ss=*(im+(dy+y)*step+dx+x);

				   if (*(im+y*step+x) <= *(im+(dy+y)*step+dx+x)) {

					   *(im+y*step+x)= 10;
					   tested= true;

                       junctions[j].x=-1;
                       junctions[j].y=-1;
                       junctions[j]= junctions[junctions.size()-1]; 
                       junctions.pop_back();
                       j--;
				   }
			   }
		   }
	   }
}

void saveJunctions(const char *name, std::vector<Junction> &junctions) {

  FILE* fptr= fopen(name, "w");
  
  for (int i=0; i<junctions.size(); i++) {

    if (junctions[i].x < 0)
      continue;

    fprintf(fptr,"%d %d %d ", junctions[i].n, junctions[i].x, junctions[i].y);

    for (int j=0; j<junctions[i].n; j++) {

      fprintf(fptr,"%f %4.2f ",  
                 180./3.1415927*junctions[i].orientation[j], junctions[i].strenght[j]);
    }

    fprintf(fptr,"\n");
  }

  fclose(fptr);
}


void loadJunctions(const char *name, std::vector<Junction> &junctions) {

  FILE* fptr= fopen(name, "r");
  int nj;
  int x,y;
  float angle, str;
  Junction *j;

  while (fscanf(fptr,"%d",&nj)==1) {

	j= new Junction(nj);  

	fscanf(fptr,"%d %d",&x, &y);

	j->x= x;
	j->y= y;

	for (int i=0; i<nj; i++) {

		fscanf(fptr,"%f %f",&angle, &str);
		j->orientation[i]= angle*3.1415927/180.;
		j->strenght[i]= str;
	}

	junctions.push_back(*j);
	delete j;
  }

  fclose(fptr);
}


void saveJunctionHistogram(const char *name, std::vector<Junction> &junctions,
						   int nangle, float awidth) {

  float astep= 2*3.1415927/nangle;
  int *hist= new int[nangle*nangle];
  float *hmin= new float[nangle];
  float *hmax= new float[nangle];
  float o1,o2;

  o1= astep/2.;
  for (int h=0; h<nangle; h++) {

	  hmin[h]= o1-awidth/2;
	  hmax[h]= o1+awidth/2;

	  o1+= astep;
  }

  for (int x=0; x<nangle; x++)
		for (int y=0; y<nangle; y++) {

				hist[x*nangle+y]= 0;
		}

  for (int i=0; i<junctions.size(); i++) {

    if (junctions[i].x < 0)
      continue;

	o1= junctions[i].orientation[0]+3.1415927;

	o2= junctions[i].orientation[1]+3.1415927;

	for (int x=0; x<nangle; x++)
		for (int y=0; y<nangle; y++) {

			if ((o1 > hmin[x] && o1 < hmax[x] || 
				 hmax[x] > 2*3.1415927 && o1 < hmax[x]-2*3.1415927 ||
				 hmin[x] < 0 && o1 > hmin[x]+2*3.1415927) 
				 &&
				(o2 > hmin[y] && o2 < hmax[y] || 
				 hmax[y] > 2*3.1415927 && o2 < hmax[y]-2*3.1415927 ||
				 hmin[y] < 0 && o2 > hmin[y]+2*3.1415927) 
				 ) {

				hist[max(x,y)*nangle+min(y,x)]++;
			} 
		}
  }

  FILE* fptr= fopen(name, "w");

  for (x=0; x<nangle; x++) {

		for (int y=0; y<nangle; y++) {

			fprintf(fptr,"%4d ", hist[x*nangle+y]);
		}

		fprintf(fptr,"\n");
  }

  fclose(fptr);
}


void drawJunctions(IplImage* image, std::vector<Junction> &junctions, int radius, int numberOfBranches) {

  int white= 0xFFFFFFFF;
  int black= 0;
  int nl= image->height - 1;
  float pi4= 3.1415927f/4.0f;

  // get circle points and edge points
  int npts;
  int** c= getCirclePoints((double)radius, &npts);

  int jx,jy;

  for (int i=0; i<junctions.size(); i++) {

    if (junctions[i].x < 0)
      continue;

    if (numberOfBranches != 0 && junctions[i].n != numberOfBranches) continue; // do not display these junctions

    for (int j=0; j<junctions[i].n; j++) {


      iplPutPixel(image, junctions[i].x, junctions[i].y, &white);

        for (int q=1; q < radius; q++) {

          jx= junctions[i].x + q*cos(junctions[i].orientation[j]);
          jx= jx < 0 ? 0 : jx;
          jx= jx < image->width ? jx : image->width - 1;

          jy= junctions[i].y + q*sin(junctions[i].orientation[j]);
          jy= jy < 0 ? 0 : jy;
          jy= jy < image->height ? jy : image->height - 1;

		  if (junctions[i].orientation[j]> 3*pi4 || junctions[i].orientation[j] < -3.*pi4 ||
			  (junctions[i].orientation[j]<pi4 && junctions[i].orientation[j] > -pi4)) {

		   iplPutPixel(image, jx, jy+1, &black); 
           iplPutPixel(image, jx, jy-1, &black); 
		   iplPutPixel(image, jx, jy+2, &black); 
           iplPutPixel(image, jx, jy-2, &black); 

		  } else {

           iplPutPixel(image, jx+1, jy, &black); 
           iplPutPixel(image, jx-1, jy, &black); 
           iplPutPixel(image, jx+2, jy, &black); 
           iplPutPixel(image, jx-2, jy, &black); 
		  }
        }

        for (q=1; q < radius; q++) {

          jx= junctions[i].x + q*cos(junctions[i].orientation[j]);
          jx= jx < 0 ? 0 : jx;
          jx= jx < image->width ? jx : image->width - 1;

          jy= junctions[i].y + q*sin(junctions[i].orientation[j]);
          jy= jy < 0 ? 0 : jy;
          jy= jy < image->height ? jy : image->height - 1;

		  if (junctions[i].orientation[j]> 3*pi4 || junctions[i].orientation[j] < -3.*pi4 ||
			  (junctions[i].orientation[j]<pi4 && junctions[i].orientation[j] > -pi4)) {

           iplPutPixel(image, jx, jy, &white); 

		  } else {

           iplPutPixel(image, jx, jy, &white); 
		  }
        }
    }

  }
}


/*
 *  Generate an image showing circles with
 *  the radial lines. To be used for testing.
 */
IplImage* junctionRadialLines(int radius) {

  int npts;
  int** c= getCirclePoints((double)radius, &npts);
  int**** pt= getLinePoints(c, npts, radius);

  int np= static_cast<int>(sqrt(npts));
  int nc= np*(2*radius+4);
  int nl= (static_cast<int>(npts/np)+1)*(2*radius+4);
  
  IplImage* res = iplCreateImageHeader(1,0,IPL_DEPTH_8U,"GRAY","G",IPL_DATA_ORDER_PIXEL,IPL_ORIGIN_TL,IPL_ALIGN_QWORD,nc,nl,NULL,NULL,NULL,NULL);
  iplAllocateImage(res, 1, 0);
  char* result= res->imageData;
  int step= res->widthStep;

  int x,y;

  for (int i=0; i<npts; i++) {


    x= (2*radius+2)*(i%np)+radius+4;
    y= (2*radius+2)*static_cast<int>(i/np)+radius+4;


    for (int r=0; r<npts; r++) {

      *(result+step*(y+c[1][r])+x+c[0][r])= 150;
    }

    for (int xx=0; xx< radius && 
                           (pt[0][i][xx][0]!=0 || pt[1][i][xx][0]!=0); xx++) {

             
        *(result+step*(y+pt[1][i][xx][0])+x+pt[0][i][xx][0])= 100+pt[0][i][xx][2];
        *(result+step*(y+pt[1][i][xx][1])+x+pt[0][i][xx][1])= 100+pt[0][i][xx][3];
    }


    *(result+step*y+x)= 255;
  }

      cvvNamedWindow( "Radius", 1 ); 
      cvvShowImage( "Radius", convertToColor(res) ); 
      cvvSaveImage("test.bmp",convertToColor(res));

  return res;
}

/*
detectJunctions()
{
   // display original image
   ipView( img, imagename, false);
		 
   // compute gaussuian derivatives
   IplImage* dx = derivex(imgf,variance);	
   IplImage* dy = derivey(imgf,variance);	
   IplImage* edge= norm(dx,dy);

   // display edge image
   IplImage* edgeImage= convertToImage(edge);
   ipView( edgeImage, "Edges", false);
   iplDeallocate( edgeImage, IPL_IMAGE_ALL );

   // apply threshold
   IplImage* edgemap= threshold(edge, thresh);

   // display binary edge map
   ipView( edgemap, "Binary Edge Map", false);

   // non max suppression
   IplImage* edgeMax= nonMax(edge, dx, dy);
   IplImage* edgeMaxBin= threshold(edgeMax, thresh);

   // display binary max edge map
   ipView( edgeMaxBin, "Max Binary Edge Map", false);

   // get junctions
   std::vector<Junction> junctions;
   IplImage* junction= judoca(edgemap, edgeMaxBin, 11, 8, junctions);
   ipView( junction, "JUDOCA junctions", false);

   // save junctions
   saveJunctions("test.junctions", junctions);

   // display junctions
//   IplImage* detected= combine(img, junction);
//   ipView( detected, "Junctions on image", true);
   drawJunctions(img,junctions,15);
   ipView( img, "Junctions on image", true);

   // deallocations
   iplDeallocate( img, IPL_IMAGE_ALL );
   iplDeallocate( junction, IPL_IMAGE_ALL );
   iplDeallocate( edgeMaxBin, IPL_IMAGE_ALL );
   iplDeallocate( edgemap,    IPL_IMAGE_ALL );
   iplDeallocate( imgf,       IPL_IMAGE_ALL );
   iplDeallocate( dx,         IPL_IMAGE_ALL );
   iplDeallocate( dy,         IPL_IMAGE_ALL );
   iplDeallocate( edge,       IPL_IMAGE_ALL );
   iplDeallocate( edgeMax,    IPL_IMAGE_ALL );
}
*/